@@ -1,5 +1,5 @@
 /* channel.c: Primitive functions for user-facing gzochid channel API
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2012 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -24,12 +24,13 @@
 #include "../session.h"
 
 #include "channel.h"
+#include "util.h"
 
 SCM_DEFINE (primitive_create_channel, "primitive-create-channel", 1, 0, 0,
 	    (SCM name), "Create a new channel with the specified name.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   char *cname = scm_to_locale_string (name);
   gzochid_channel *channel = gzochid_channel_create (context, cname);
   gzochid_data_managed_reference *scm_reference = 
@@ -38,6 +39,8 @@ SCM_DEFINE (primitive_create_channel, "primitive-create-channel", 1, 0, 0,
 
   gzochid_data_dereference (scm_reference);
 
+  gzochid_api_check_rollback ();
+
   return (SCM) scm_reference->obj;
 }
 
@@ -45,28 +48,31 @@ SCM_DEFINE (primitive_get_channel, "primitive-get-channel", 1, 0, 0,
 	    (SCM name), "Retrieves the channel with the specified name.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   char *cname = scm_to_locale_string (name);
   gzochid_channel *channel = gzochid_channel_get (context, cname);
-
+  SCM ret = SCM_BOOL_F;
+  
   free (cname);
-  if (channel == NULL)
-    return SCM_BOOL_F;
-  else
+  if (channel != NULL)
     {
       gzochid_data_managed_reference *scm_reference = 
 	gzochid_data_create_reference_to_oid 
 	(context, &gzochid_scheme_data_serialization, channel->scm_oid);
       gzochid_data_dereference (scm_reference);
-      return (SCM) scm_reference->obj;
+      ret = (SCM) scm_reference->obj;
     }
+
+  gzochid_api_check_rollback ();
+
+  return ret;
 }
 
 SCM_DEFINE (primitive_join_channel, "primitive-join-channel", 2, 0, 0,
 	    (SCM channel, SCM session), "Add client session to a channel.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *channel_reference = NULL;
   gzochid_data_managed_reference *session_reference = NULL;
   mpz_t channel_oid, session_oid;
@@ -89,6 +95,8 @@ SCM_DEFINE (primitive_join_channel, "primitive-join-channel", 2, 0, 0,
      (gzochid_channel *) channel_reference->obj, 
      (gzochid_client_session *) session_reference->obj);
 
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
@@ -97,7 +105,7 @@ SCM_DEFINE (primitive_leave_channel, "primitive-leave-channel", 2, 0, 0,
 	    "Remove a client session from a channel.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *channel_reference = NULL;
   gzochid_data_managed_reference *session_reference = NULL;
   mpz_t channel_oid, session_oid;
@@ -119,6 +127,9 @@ SCM_DEFINE (primitive_leave_channel, "primitive-leave-channel", 2, 0, 0,
     (context, 
      (gzochid_channel *) channel_reference->obj, 
      (gzochid_client_session *) session_reference->obj);
+
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
@@ -127,7 +138,7 @@ SCM_DEFINE (primitive_send_channel_message, "primitive-send-channel-message",
 	    "Send a message to the sessions that belong to a channel.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
 
   short len = (short) SCM_BYTEVECTOR_LENGTH (bv);
   unsigned char *msg = (unsigned char *) SCM_BYTEVECTOR_CONTENTS (bv);
@@ -145,6 +156,9 @@ SCM_DEFINE (primitive_send_channel_message, "primitive-send-channel-message",
 
   gzochid_channel_send 
     (context, (gzochid_channel *) channel_reference->obj, msg, len);
+
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
@@ -152,7 +166,7 @@ SCM_DEFINE (primitive_close_channel, "primitive-close-channel", 1, 0, 0,
 	    (SCM channel, SCM msg), "Shut down a channel.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *channel_reference = NULL;
   mpz_t channel_oid;
 
@@ -163,6 +177,9 @@ SCM_DEFINE (primitive_close_channel, "primitive-close-channel", 1, 0, 0,
   gzochid_data_dereference (channel_reference);
 
   gzochid_channel_close (context, (gzochid_channel *) channel_reference->obj);
+
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
