@@ -17,32 +17,42 @@
 
 #include <libguile.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "../app.h"
 #include "../data.h"
+#include "../reloc.h"
 #include "../scheme.h"
 
 #include "data.h"
+#include "util.h"
 
 SCM_DEFINE (primitive_create_reference, "primitive-create-reference", 1, 0, 0, 
 	    (SCM obj), "Create a managed reference.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
+  gzochid_scm_location_info *obj_loc = gzochid_scm_location_get (context, obj);
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, obj);
-  
+    (context, &gzochid_scm_location_aware_serialization, obj_loc);
+  SCM ret = SCM_BOOL_F;
+
   scm_gc_protect_object (obj);
 
-  return gzochid_scheme_create_managed_reference (reference);
+  ret = gzochid_scheme_create_managed_reference (reference);
+  
+  gzochid_api_check_rollback ();
+
+  return ret;
 }
 
 SCM_DEFINE (primitive_dereference, "primitive-dereference", 1, 0, 0, (SCM ref),
 	    "Dereference a managed reference.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *reference = NULL;
+  SCM ret = SCM_BOOL_F;
   mpz_t oid;
   
   mpz_init (oid);
@@ -50,101 +60,145 @@ SCM_DEFINE (primitive_dereference, "primitive-dereference", 1, 0, 0, (SCM ref),
   gzochid_scheme_managed_reference_oid (ref, oid);
 
   reference = gzochid_data_create_reference_to_oid 
-    (context, &gzochid_scheme_data_serialization, oid);
+    (context, &gzochid_scm_location_aware_serialization, oid);
   gzochid_data_dereference (reference);
 
-  if (reference->obj == NULL)
-    return gzochid_scheme_r6rs_raise 
+  if (reference->obj == NULL
+      || reference->state == GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED)
+    gzochid_scheme_r6rs_raise 
       (gzochid_scheme_make_object_removed_condition ());
-  else return reference->obj;
+  else 
+    {
+      gzochid_scm_location_info *location = 
+	(gzochid_scm_location_info *) reference->obj;
+      ret = gzochid_scm_location_resolve (context, location);
+    }
+
+  gzochid_api_check_rollback ();
+
+  return ret;
+}
+
+static char *prefix_name (char *name)
+{
+  int name_len = strlen (name);
+  char *prefixed_name = malloc (sizeof (char) * (name_len + 3));
+  
+  prefixed_name = strncpy (prefixed_name, "o.", 3);
+  prefixed_name = strncat (prefixed_name, name, name_len);
+
+  return prefixed_name;
 }
 
 SCM_DEFINE (primitive_get_binding, "primitive-get-binding", 1, 0, 0, (SCM name),
 	    "Retrieve a managed record bound to a name.")
 {
   gzochid_application_context *context =
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
+  
   char *cname = scm_to_locale_string (name);
+  char *prefixed_name = prefix_name (cname);
+  
+  SCM ret = SCM_BOOL_F;
 
-  if (!gzochid_data_binding_exists (context, cname))
+  if (!gzochid_data_binding_exists (context, prefixed_name))
     {
       SCM cond = gzochid_scheme_make_name_not_bound_condition (cname);
 
       free (cname);
+      free (prefixed_name);
 
-      return gzochid_scheme_r6rs_raise (cond);
+      gzochid_scheme_r6rs_raise (cond);
     }
   else 
     {
-      SCM obj = (SCM) gzochid_data_get_binding 
-	(context, cname, &gzochid_scheme_data_serialization);
+      gzochid_scm_location_info *location = 
+	(gzochid_scm_location_info *) gzochid_data_get_binding 
+	(context, prefixed_name, &gzochid_scm_location_aware_serialization);
+      ret = gzochid_scm_location_resolve (context, location);
 
       free (cname);
-
-      return obj;
+      free (prefixed_name);
     }
+
+  gzochid_api_check_rollback ();
+  return ret;
 }
 
 SCM_DEFINE (primitive_set_binding_x, "primitive-set-binding!", 2, 0, 0,
 	    (SCM name, SCM obj), "Bind a managed record to a name.")
 {
   gzochid_application_context *context =
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   char *cname = scm_to_locale_string (name);
+  char *prefixed_name = prefix_name (cname);
   
-  if (gzochid_data_binding_exists (context, cname))
+  if (gzochid_data_binding_exists (context, prefixed_name))
     {
       SCM cond = gzochid_scheme_make_name_exists_condition (cname);
 
       free (cname);
+      free (prefixed_name);
 
-      return gzochid_scheme_r6rs_raise (cond);
+      gzochid_scheme_r6rs_raise (cond);
     }
   else
     {
+      gzochid_scm_location_info *obj_loc = 
+	gzochid_scm_location_get (context, obj);
+
       gzochid_data_set_binding 
-	(context, cname, &gzochid_scheme_data_serialization, obj);
+	(context, prefixed_name, &gzochid_scm_location_aware_serialization, 
+	 obj_loc);
       
       free (cname);
-      
-      return SCM_UNSPECIFIED;
+      free (prefixed_name);
     }
+
+  gzochid_api_check_rollback ();
+  return SCM_UNSPECIFIED;
 }
 
 SCM_DEFINE (primitive_remove_binding_x, "primitive-remove-binding!", 1, 0, 0,
 	    (SCM name), "Remove the binding for a name.")
 {
   gzochid_application_context *context =
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   char *cname = scm_to_locale_string (name);
+  char *prefixed_name = prefix_name (cname);
   
-  if (!gzochid_data_binding_exists (context, cname))
+  if (!gzochid_data_binding_exists (context, prefixed_name))
     {
       SCM cond = gzochid_scheme_make_name_not_bound_condition (cname);
 
       free (cname);
+      free (prefixed_name);
 
-      return gzochid_scheme_r6rs_raise (cond);
+      gzochid_scheme_r6rs_raise (cond);
     }
   else
     {
-      gzochid_data_remove_binding (context, cname);
+      gzochid_data_remove_binding (context, prefixed_name);
 
       free (cname);
-  
-      return SCM_UNSPECIFIED;
+      free (prefixed_name);
     }
+
+  gzochid_api_check_rollback ();
+  return SCM_UNSPECIFIED;
 }
 
 SCM_DEFINE (primitive_remove_object_x, "primitive-remove-object!", 1, 0, 0,
 	    (SCM obj), "Remove a managed record from the data store.")
 {
   gzochid_application_context *context =
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
+  gzochid_scm_location_info *obj_loc = gzochid_scm_location_get (context, obj);
   gzochid_data_managed_reference *reference = gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, obj);
+    (context, &gzochid_scm_location_aware_serialization, obj_loc);
 
   gzochid_data_remove_object (reference);
+  gzochid_api_check_rollback ();
 
   return SCM_UNSPECIFIED;
 }
@@ -152,9 +206,14 @@ SCM_DEFINE (primitive_remove_object_x, "primitive-remove-object!", 1, 0, 0,
 SCM_DEFINE (primitive_mark_for_write_x, "primitive-mark-for-write!", 1, 0, 0,
 	    (SCM obj), "Mark a managed record that has been modified.")
 {
+  gzochid_application_context *context =
+    gzochid_api_ensure_current_application_context ();
+  gzochid_scm_location_info *obj_loc = gzochid_scm_location_get (context, obj);
+
   gzochid_data_mark 
-    (gzochid_scheme_current_application_context (), 
-     &gzochid_scheme_data_serialization, obj);
+    (context, &gzochid_scm_location_aware_serialization, obj_loc);
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
