@@ -1,5 +1,5 @@
 /* session.c: Primitive functions for user-facing gzochid client session API
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2012 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,12 +23,13 @@
 #include "../scheme.h"
 
 #include "session.h"
+#include "util.h"
 
 SCM_DEFINE (primitive_send_message, "primitive-send-message", 2, 0, 0,
 	    (SCM session, SCM msg), "Send a message to a client session.")
 {
   gzochid_application_context *context =
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *reference = NULL;
   
   short len = (short) SCM_BYTEVECTOR_LENGTH (msg);
@@ -45,6 +46,9 @@ SCM_DEFINE (primitive_send_message, "primitive-send-message", 2, 0, 0,
     (context, (gzochid_client_session *) reference->obj, payload, len);
 
   mpz_clear (c_oid);
+
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
@@ -52,7 +56,7 @@ SCM_DEFINE (primitive_disconnect, "primitive-disconnect", 1, 0, 0,
 	    (SCM session), "Disconnect a client session.")
 {
   gzochid_application_context *context =
-    gzochid_scheme_current_application_context ();
+    gzochid_api_ensure_current_application_context ();
   gzochid_data_managed_reference *reference = NULL;  
   mpz_t c_oid;
 
@@ -66,6 +70,9 @@ SCM_DEFINE (primitive_disconnect, "primitive-disconnect", 1, 0, 0,
     (context, (gzochid_client_session *) reference->obj);
 
   mpz_clear (c_oid);
+
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
