@@ -23,13 +23,14 @@
 #include "../task.h"
 
 #include "task.h"
+#include "util.h"
 
 SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 2, 0, 0, 
 	    (SCM callback, SCM delay), "Schedule a durable task to run.")
 {
   gzochid_application_context *context = 
-    gzochid_scheme_current_application_context (); 
-  gzochid_auth_identity *identity = gzochid_scheme_current_identity ();
+    gzochid_api_ensure_current_application_context (); 
+  gzochid_auth_identity *identity = gzochid_get_current_identity ();
 
   gzochid_application_task *scheme_task = gzochid_application_task_new 
     (context, identity, gzochid_scheme_application_worker, callback);
@@ -38,6 +39,8 @@ SCM_DEFINE (primitive_schedule_task, "primitive-schedule-task", 2, 0, 0,
   gzochid_schedule_delayed_durable_task 
     (context, identity, scheme_task, &gzochid_scheme_task_serialization, d);
 
+  gzochid_api_check_rollback ();
+
   return SCM_UNSPECIFIED;
 }
 
