@@ -1,5 +1,5 @@
 /* app.c: Application context routines for gzochid
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2012 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -34,6 +34,9 @@
 #include "tx.h"
 #include "util.h"
 
+static GStaticPrivate thread_application_context_key = G_STATIC_PRIVATE_INIT;
+static GStaticPrivate thread_identity_key = G_STATIC_PRIVATE_INIT;
+
 static void initialize_async (gpointer data, gpointer user_data)
 {
   gzochid_context *context = (gzochid_context *) data;
@@ -49,9 +52,9 @@ static void initialize_data (int from_state, int to_state, gpointer user_data)
 
   char *data_dir = g_strconcat 
     (game_context->work_dir, "/", app_context->descriptor->name, NULL);
-  char *meta_db = g_strconcat (data_dir, "/meta.db", NULL);
-  char *oids_db = g_strconcat (data_dir, "/oids.db", NULL);
-  char *names_db = g_strconcat (data_dir, "/names.db", NULL);
+  char *meta_db = g_strconcat (data_dir, "/meta", NULL);
+  char *oids_db = g_strconcat (data_dir, "/oids", NULL);
+  char *names_db = g_strconcat (data_dir, "/names", NULL);
 
   if (!g_file_test (data_dir, G_FILE_TEST_EXISTS))
     {
@@ -88,38 +91,6 @@ static void initialize_complete
     (game_context->pool, initialize_async, user_data, NULL);
 }
 
-static void initialized_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
-{
-  GHashTable *properties = (GHashTable *) data;
-  gzochid_data_managed_reference *callback_reference = NULL;
-  SCM callback = gzochid_scheme_create_callback 
-    (context->descriptor->initialized, NULL);
-
-  callback_reference = gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, callback);
-
-  gzochid_scheme_invoke 
-    (context,
-     identity,
-     "gzochi:execute-initialized",
-     g_list_append
-     (g_list_append
-      (g_list_append (NULL, "gzochi"), "private"), "app"),
-     scm_list_2 (callback,
-		 gzochid_scheme_ghashtable_to_hashtable 
-		 (properties, 
-		  gzochid_scheme_string_hash,
-		  gzochid_scheme_string_equiv,
-		  (SCM (*) (gpointer)) scm_from_locale_string,
-		  (SCM (*) (gpointer)) scm_from_locale_string)),
-     SCM_BOOL_F);
-
-  gzochid_data_set_binding_to_oid 
-    (context, "s.initializer", callback_reference->oid);
-}
-
 static void transactional_task_worker (gpointer data)
 {
   void **args = (void **) data;
@@ -169,26 +140,19 @@ static void run_async_transactional (gpointer data)
 
   if (persisted_callback == NULL)
     {
-      gzochid_game_context *game_context = 
-	(gzochid_game_context *) ((gzochid_context *) context)->parent;
-
-      gzochid_transactional_application_task transactional_task;
       gzochid_application_task application_task;
       gzochid_task task;
 
-      transactional_task.worker = initialized_worker;
-      transactional_task.data = context->descriptor->properties;
-      
-      application_task.worker = gzochid_application_transactional_task_worker;
+      application_task.worker = gzochid_scheme_application_initialized_worker;
       application_task.context = context;
       application_task.identity = system_identity;
-      application_task.data = &transactional_task;
+      application_task.data = context->descriptor->properties;
 
       task.worker = gzochid_application_task_thread_worker;
       task.data = &application_task;
       gettimeofday (&task.target_execution_time, NULL);
 
-      gzochid_schedule_run_task (game_context->task_queue, &task);
+      gzochid_schedule_execute_task (&task);
     }
   else 
     {
@@ -344,92 +308,6 @@ void gzochid_application_context_init
   gzochid_context_init ((gzochid_context *) context, parent, fsm);
 }
 
-static gzochid_application_callback *scm_to_callback 
-(gzochid_application_context *context, SCM scm_callback)
-{
-  GList *module = gzochid_scheme_callback_module (scm_callback);
-  char *procedure = gzochid_scheme_callback_procedure (scm_callback);
-  gzochid_data_managed_reference *reference = 
-    gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, scm_callback);
-
-  mpz_t oid;
-  
-  mpz_init (oid);
-  mpz_set (oid, reference->oid);      
-    
-  return gzochid_application_callback_new (procedure, module, oid);
-}
-
-static void client_logged_in_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
-{
-  mpz_t session_oid;
-  char *session_oid_str = (char *) data;
-  gzochid_client_session *session = NULL;
-  gzochid_data_managed_reference *session_reference = NULL;
-
-  SCM scm_session = SCM_BOOL_F;
-  gzochid_data_managed_reference *session_scm_reference = NULL;
-
-  SCM cb = SCM_BOOL_F;
-  gzochid_data_managed_reference *callback_reference = NULL;
-
-  SCM handler = SCM_BOOL_F;
-
-  mpz_init (session_oid);
-  mpz_set_str (session_oid, session_oid_str, 16);
-  session_reference = gzochid_data_create_reference_to_oid
-    (context, &gzochid_client_session_serialization, session_oid);
-  mpz_clear (session_oid);
-
-  gzochid_data_dereference (session_reference);
-  session = (gzochid_client_session *) session_reference->obj;
-
-  scm_session = gzochid_scheme_create_client_session 
-    (session, session_reference->oid);
-  session_scm_reference = 
-    gzochid_data_create_reference
-    (context, &gzochid_scheme_data_serialization, scm_session);
-
-  cb = gzochid_scheme_create_callback (context->descriptor->logged_in, NULL);
-  callback_reference = gzochid_data_create_reference 
-    (context, &gzochid_scheme_data_serialization, cb);
-
-  handler = gzochid_scheme_invoke 
-    (context,
-     identity,
-     "gzochi:execute-logged-in",
-     g_list_append
-     (g_list_append
-      (g_list_append (NULL, "gzochi"), "private"), "app"),
-     scm_list_2 ((SCM) callback_reference->obj, 
-		 (SCM) session_scm_reference->obj), 
-     SCM_BOOL_F);
-  
-  if (scm_is_false (handler))
-    gzochid_client_session_send_login_failure (context, session);
-  else
-    {
-      gzochid_client_session_handler *session_handler = 
-	malloc (sizeof (gzochid_client_session_handler));
-
-      session_handler->received_message = 
-	scm_to_callback
-	(context, gzochid_scheme_handler_received_message (handler));
-      session_handler->disconnected =
-	scm_to_callback 
-	(context, gzochid_scheme_handler_disconnected (handler));
-
-      session->handler = session_handler;
-
-      gzochid_data_mark 
-	(context, &gzochid_client_session_serialization, session);
-      gzochid_client_session_send_login_success (context, session);
-    }
-}
-
 void gzochid_application_client_logged_in
 (gzochid_application_context *context, gzochid_protocol_client *client)
 {
@@ -452,7 +330,7 @@ void gzochid_application_client_logged_in
   g_hash_table_insert (context->oids_to_clients, session_oid_str, client);
   g_hash_table_insert (context->clients_to_oids, client, session_oid_str);
 
-  transactional_task.worker = client_logged_in_worker;
+  transactional_task.worker = gzochid_scheme_application_logged_in_worker;
   transactional_task.data = session_oid_str;
 
   application_task.worker = gzochid_application_transactional_task_worker;
@@ -467,42 +345,6 @@ void gzochid_application_client_logged_in
   gzochid_schedule_run_task (game_context->task_queue, &task);
 }
 
-static void client_disconnected_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer ptr)
-{
-  gzochid_client_session *session = NULL;
-  gzochid_data_managed_reference *session_reference = NULL;
-  gzochid_data_managed_reference *callback_reference = NULL;
-
-  GList *gpa = g_list_append
-    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
-
-  mpz_t session_oid;
-  mpz_init (session_oid);
-  mpz_set_str (session_oid, (char *) ptr, 16);
-
-  session_reference = gzochid_data_create_reference_to_oid 
-    (context, &gzochid_client_session_serialization, session_oid);
-  gzochid_data_dereference (session_reference);
-  session = (gzochid_client_session *) session_reference->obj;
-
-  callback_reference =
-    gzochid_data_create_reference_to_oid
-    (context, &gzochid_scheme_data_serialization, 
-     session->handler->disconnected->scm_oid);
-  gzochid_data_dereference (callback_reference);
-      
-  gzochid_scheme_invoke
-    (context, 
-     identity,
-     "gzochi:execute-disconnected", gpa,
-     scm_list_1 ((SCM) callback_reference->obj),
-     SCM_BOOL_F);
-
-  g_list_free (gpa);
-}
-
 void gzochid_application_client_disconnected
 (gzochid_application_context *context, gzochid_protocol_client *client)
 {
@@ -522,7 +364,7 @@ void gzochid_application_client_disconnected
     {
       gzochid_transactional_application_task *transactional_task =
 	gzochid_transactional_application_task_new
-	(client_disconnected_worker, session_oid_str);
+	(gzochid_scheme_application_disconnected_worker, session_oid_str);
 
       gzochid_application_task *application_task =
 	gzochid_application_task_new
@@ -544,51 +386,6 @@ void gzochid_application_client_disconnected
     }
 }
 
-static void client_received_message_worker 
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer ptr)
-{
-  gzochid_client_session *session = NULL;
-  gzochid_data_managed_reference *session_reference = NULL;
-  gzochid_data_managed_reference *callback_reference = NULL;
-
-  SCM bv = SCM_BOOL_F;
-  GList *gpa = g_list_append
-    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
-
-  unsigned char *message = NULL;
-  short *message_len_ptr = NULL;
-  void **data = (void **) ptr;
-
-  mpz_t session_oid;
-  mpz_init (session_oid);
-  mpz_set_str (session_oid, (char *) data[0], 16);
-
-  message = (unsigned char *) data[1];
-  message_len_ptr = (short *) data[2];
-
-  session_reference = gzochid_data_create_reference_to_oid 
-    (context, &gzochid_client_session_serialization, session_oid);
-  gzochid_data_dereference (session_reference);
-  session = (gzochid_client_session *) session_reference->obj;
-
-  bv = gzochid_scheme_create_bytevector (message, *message_len_ptr);
-  callback_reference =
-    gzochid_data_create_reference_to_oid
-    (context, &gzochid_scheme_data_serialization, 
-     session->handler->received_message->scm_oid);
-  gzochid_data_dereference (callback_reference);
-      
-  gzochid_scheme_invoke
-    (context, 
-     identity,
-     "gzochi:execute-received-message", gpa,
-     scm_list_2 ((SCM) callback_reference->obj, bv),
-     SCM_BOOL_F);
-
-  g_list_free (gpa);
-}
-
 gzochid_application_worker_serialization received_message_worker_serialization =
   { NULL, NULL };
 
@@ -629,7 +426,8 @@ void gzochid_application_session_received_message
       data[1] = msg;
       data[2] = &len;
       
-      transactional_task.worker = client_received_message_worker;
+      transactional_task.worker = 
+	gzochid_scheme_application_received_message_worker;
       transactional_task.data = data;
       
       application_task.worker = gzochid_application_transactional_task_worker;
@@ -650,3 +448,49 @@ void gzochid_application_channel_message_received
  char *channel, unsigned char *msg, short len)
 {
 }
+
+void *gzochid_with_application_context 
+(gzochid_application_context *context, gzochid_auth_identity *identity,
+ void *(*worker) (gpointer), gpointer data)
+{
+  gpointer ret = NULL;
+  gboolean private_needs_context = 
+    g_static_private_get (&thread_application_context_key) == NULL;
+  SCM application_root_fluid = 
+    scm_variable_ref
+    (scm_c_module_lookup 
+     (scm_c_resolve_module("gzochi app"), "%gzochi:application-root"));
+
+  if (private_needs_context)
+    {
+      g_static_private_set (&thread_application_context_key, context, NULL);
+      g_static_private_set (&thread_identity_key, identity, NULL);
+      
+      scm_fluid_set_x
+	(application_root_fluid, 
+	 scm_from_locale_string (context->descriptor->deployment_root));
+    }
+  
+  ret = worker (data);
+
+  if (private_needs_context)
+    {
+      g_static_private_set (&thread_application_context_key, NULL, NULL);
+      g_static_private_set (&thread_identity_key, NULL, NULL);
+
+      scm_fluid_set_x (application_root_fluid, SCM_UNSPECIFIED);
+    }
+
+  return ret;
+}
+
+gzochid_application_context *gzochid_get_current_application_context (void)
+{
+  return (gzochid_application_context *) 
+    g_static_private_get (&thread_application_context_key);
+}
+
+gzochid_auth_identity *gzochid_get_current_identity (void)
+{
+  return (gzochid_auth_identity *) g_static_private_get (&thread_identity_key);
+}
