@@ -102,19 +102,10 @@ static void flush_reference (gpointer data, gpointer user_data)
 
   switch (reference->state)
     {
-    case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY: break;
-
-    case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED:
+    case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY:
     case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY:
-
-      oid_str = mpz_get_str (NULL, 16, reference->oid);
-
-      gzochid_debug ("Removing reference '%s'.", oid_str);      
-      gzochid_storage_transaction_delete
-	(context->oids_transaction, oid_str, strlen (oid_str) + 1);
-
       break;
-      
+
     case GZOCHID_MANAGED_REFERENCE_STATE_NEW:
     case GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED:
       
@@ -133,6 +124,7 @@ static void flush_reference (gpointer data, gpointer user_data)
       g_string_free (out, FALSE);
 
       break;
+    case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED:
     case GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED: break;
     default:
       assert (1 == 0);
@@ -198,7 +190,7 @@ static gzochid_data_oid_block *create_oid_block
 {
   char next_oid_key[] = { NEXT_OID_KEY };
   char *next_oid_value = NULL;
-  int next_oid_value_len = 0;
+  size_t next_oid_value_len = 0;
 
   gzochid_data_oid_block *block = calloc (1, sizeof (gzochid_data_oid_block));
 
@@ -258,6 +250,8 @@ static void get_binding
 {
   char *oid_str = gzochid_storage_transaction_get 
     (context->names_transaction, name, strlen (name) + 1, NULL);
+  if (context->names_transaction->rollback)
+    gzochid_transaction_mark_for_rollback (&data_participant);
 
   if (oid_str != NULL)
     {
@@ -275,6 +269,9 @@ static void set_binding
     (context->names_transaction, name, strlen (name) + 1, oid_str, 
      strlen (oid_str) + 1);
 
+  if (context->names_transaction->rollback)
+    gzochid_transaction_mark_for_rollback (&data_participant);
+
   free (oid_str);
 }
 
@@ -388,19 +385,22 @@ void dereference
 (gzochid_data_transaction_context *context, 
  gzochid_data_managed_reference *reference)
 {
-  int data_len = 0;
+  size_t data_len = 0;
   char *oid_str = NULL; 
   char *data = NULL; 
   GString *in = NULL;
 
-  if (reference->obj != NULL)
+  if (reference->obj != NULL 
+      || reference->state == GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY)
     return;
 
   oid_str = mpz_get_str (NULL, 16, reference->oid);
   gzochid_debug ("Retrieving data for reference '%s'.", oid_str);
 
-  data = gzochid_storage_transaction_get 
+  data = gzochid_storage_transaction_get
     (context->oids_transaction, oid_str, strlen (oid_str) + 1, &data_len);
+  if (context->oids_transaction->rollback)
+    gzochid_transaction_mark_for_rollback (&data_participant);
 
   if (data == NULL)
     {
@@ -422,13 +422,36 @@ void dereference
     }
 }
 
-static void remove_object 
+static void remove_object (gzochid_data_transaction_context *context, mpz_t oid)
+{
+  char *oid_str = mpz_get_str (NULL, 16, oid);
+  gzochid_debug ("Removing reference '%s'.", oid_str);      
+  gzochid_storage_transaction_delete 
+    (context->oids_transaction, oid_str, strlen (oid_str) + 1);
+  free (oid_str);
+}
+
+static void remove_reference 
 (gzochid_data_transaction_context *context, 
  gzochid_data_managed_reference *reference)
 {
-  if (reference->state == GZOCHID_MANAGED_REFERENCE_STATE_EMPTY)
-    reference->state = GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY;
-  else reference->state = GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED;
+  switch (reference->state)
+    {
+    case GZOCHID_MANAGED_REFERENCE_STATE_EMPTY:
+      
+      remove_object (context, reference->oid);
+      reference->state = GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_EMPTY;
+      break;
+
+    case GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED:
+    case GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED:
+
+      remove_object (context, reference->oid);
+
+    case GZOCHID_MANAGED_REFERENCE_STATE_NEW:
+      reference->state = GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED;
+    default: break;
+    }
 }
 
 static void join_transaction (gzochid_application_context *context)
@@ -486,11 +509,16 @@ char *gzochid_data_next_binding_oid
   tx_context = gzochid_transaction_context (&data_participant);
   next_key = gzochid_storage_transaction_next_key 
     (tx_context->names_transaction, key, strlen (key) + 1, NULL);
+  if (tx_context->names_transaction->rollback)
+    gzochid_transaction_mark_for_rollback (&data_participant);
 
   if (next_key != NULL)
     {
       char *next_value = gzochid_storage_transaction_get 
 	(tx_context->names_transaction, next_key, strlen (next_key) + 1, NULL);
+      if (tx_context->names_transaction->rollback)
+	gzochid_transaction_mark_for_rollback (&data_participant);
+
       mpz_set_str (oid, next_value, 16);
       free (next_value);
       return next_key;
@@ -530,8 +558,8 @@ void gzochid_data_remove_binding
 
   gzochid_storage_transaction_delete 
     (tx_context->names_transaction, name, strlen (name) + 1);
-  gzochid_storage_transaction_delete
-    (tx_context->oids_transaction, oid_str, strlen (oid_str) + 1);
+  if (tx_context->names_transaction->rollback)
+    gzochid_transaction_mark_for_rollback (&data_participant);
 
   free (oid_str);
 }
@@ -655,18 +683,36 @@ void gzochid_data_dereference (gzochid_data_managed_reference *reference)
 void gzochid_data_remove_object (gzochid_data_managed_reference *reference)
 {
   join_transaction (reference->context);
-  remove_object (gzochid_transaction_context (&data_participant), reference);
+  remove_reference (gzochid_transaction_context (&data_participant), reference);
 }
 
 void gzochid_data_mark 
 (gzochid_application_context *context, gzochid_io_serialization *serialization,
  void *ptr)
 {
-  gzochid_data_managed_reference *reference =
-    get_reference_by_ptr (context, ptr, serialization);
+  gzochid_data_managed_reference *reference = NULL;
+  join_transaction (context);
+
+  reference = get_reference_by_ptr (context, ptr, serialization);
 
   if (reference->state != GZOCHID_MANAGED_REFERENCE_STATE_NEW)
-    reference->state = GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED;
+    {
+      char *data = NULL;
+      char *oid_str = mpz_get_str (NULL, 16, reference->oid); 
+      gzochid_data_transaction_context *tx_context = 
+	gzochid_transaction_context (&data_participant);
+
+      reference->state = GZOCHID_MANAGED_REFERENCE_STATE_MODIFIED;
+      gzochid_debug ("Marking reference '%s' for update.", oid_str);
+
+      data = gzochid_storage_transaction_get_for_update
+	(tx_context->oids_transaction, oid_str, strlen (oid_str) + 1, NULL);
+      if (tx_context->oids_transaction->rollback)
+	gzochid_transaction_mark_for_rollback (&data_participant);
+
+      free (oid_str);
+      free (data);
+    }
 }
 
 typedef struct _gzochid_persistence_task_data
