@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2012 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -169,7 +169,11 @@ void gzochid_game_context_init
   gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_RUNNING, "RUNNING");
   gzochid_fsm_add_state (fsm, GZOCHID_GAME_STATE_STOPPED, "STOPPED");
 
-  context->pool = gzochid_thread_pool_new (context, 3, FALSE, NULL);
+  context->pool = gzochid_thread_pool_new 
+    (context, 
+     gzochid_config_to_int 
+     (g_hash_table_lookup (config, "thread_pool.max_threads"), 4), 
+     TRUE, NULL);
   context->task_queue = gzochid_task_queue_new (context->pool);
 
   gzochid_schedule_task_queue_start (context->task_queue);
@@ -218,3 +222,8 @@ gzochid_application_context *gzochid_game_context_lookup_application
 {
   return g_hash_table_lookup (context->applications, name);
 }
+
+GList *gzochid_game_context_get_applications (gzochid_game_context *context)
+{
+  return g_hash_table_get_values (context->applications);
+}
