@@ -1,5 +1,5 @@
 /* gzochid.c: Main server bootstrapping routines for gzochid
- * Copyright (C) 2011 Julian Graham
+ * Copyright (C) 2012 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include <config.h>
 #include <getopt.h>
+#include <glib.h>
 #include <locale.h>
 #include <libintl.h>
 #include <stdio.h>
@@ -28,8 +29,14 @@
 #include "fsm.h"
 #include "game.h"
 #include "gzochid.h"
+#include "log.h"
 #include "threads.h"
 
+/* Need to move #include for syslog down here because we have other depedencies
+   like Serveez that attempt to redefine some of its `LOG_' #defines. */
+
+#include <syslog.h>
+
 #define _(String) gettext (String)
 
 #define Q(x) #x
@@ -54,27 +61,54 @@ static void initialize_async (gpointer data, gpointer user_data)
   GKeyFile *key_file = g_key_file_new ();
   GHashTable *admin_config = NULL;
   GHashTable *game_config = NULL;  
-  
-  server_context->admin_context = gzochid_admin_context_new ();
-  server_context->game_context = gzochid_game_context_new ();
+  GHashTable *log_config = NULL;
+
+  server_context->admin_context = 
+    (gzochid_context *) gzochid_admin_context_new ();
+  server_context->game_context = 
+    (gzochid_context *) gzochid_game_context_new ();
 
   g_key_file_load_from_file 
     (key_file, QUOTE(GZOCHID_CONF_LOCATION), G_KEY_FILE_NONE, NULL);
 
   admin_config = gzochid_config_keyfile_extract_config (key_file, "admin");
   game_config = gzochid_config_keyfile_extract_config (key_file, "game");
+  log_config = gzochid_config_keyfile_extract_config (key_file, "log");
+
+  if (g_hash_table_contains (log_config, "priority.threshold")) 
+    {
+      char *threshold = g_hash_table_lookup (log_config, "priority.threshold");
+
+      if (g_ascii_strncasecmp (threshold, "EMERG", 5) == 0)
+	gzochid_set_log_threshold (LOG_EMERG);
+      else if (g_ascii_strncasecmp (threshold, "ALERT", 5) == 0)
+	gzochid_set_log_threshold (LOG_ALERT);
+      else if (g_ascii_strncasecmp (threshold, "CRIT", 4) == 0)
+	gzochid_set_log_threshold (LOG_CRIT);
+      else if (g_ascii_strncasecmp (threshold, "ERR", 3) == 0)
+	gzochid_set_log_threshold (LOG_ERR);
+      else if (g_ascii_strncasecmp (threshold, "WARNING", 7) == 0)
+	gzochid_set_log_threshold (LOG_WARNING);
+      else if (g_ascii_strncasecmp (threshold, "NOTICE", 6) == 0)
+	gzochid_set_log_threshold (LOG_NOTICE);
+      else if (g_ascii_strncasecmp (threshold, "INFO", 4) == 0)
+	gzochid_set_log_threshold (LOG_INFO);
+      else if (g_ascii_strncasecmp (threshold, "DEBUG", 5) == 0)
+	gzochid_set_log_threshold (LOG_DEBUG);
+    }
+  else gzochid_set_log_threshold (LOG_INFO);
 
   gzochid_admin_context_init 
-    (server_context->admin_context, context, admin_config);
+    ((gzochid_admin_context *) server_context->admin_context, context, 
+     admin_config);
   gzochid_game_context_init 
-    (server_context->game_context, context, game_config);
+    ((gzochid_game_context *) server_context->game_context, context, 
+     game_config);
 
   gzochid_context_until 
-    ((gzochid_context *) server_context->admin_context, 
-     GZOCHID_ADMIN_STATE_RUNNING);
+    (server_context->admin_context, GZOCHID_ADMIN_STATE_RUNNING);
   gzochid_context_until 
-    ((gzochid_context *) server_context->game_context, 
-     GZOCHID_GAME_STATE_RUNNING);
+    (server_context->game_context, GZOCHID_GAME_STATE_RUNNING);
 
   gzochid_fsm_to_state (context->fsm, GZOCHID_STATE_RUNNING);
 }
@@ -98,7 +132,7 @@ void gzochid_server_context_init (gzochid_server_context *context)
   gzochid_fsm_add_state (fsm, GZOCHID_STATE_RUNNING, "RUNNING");
   gzochid_fsm_add_state (fsm, GZOCHID_STATE_STOPPED, "STOPPED");
 
-  context->pool = gzochid_thread_pool_new (context, -1, FALSE, NULL);
+  context->pool = gzochid_thread_pool_new (context, 1, TRUE, NULL);
 
   gzochid_fsm_on_enter 
     (fsm, GZOCHID_STATE_INITIALIZING, initialize, context);
