@@ -26,19 +26,20 @@
 #include "app.h"
 #include "auth.h"
 #include "data.h"
+#include "guile.h"
 #include "io.h"
+#include "reloc.h"
 #include "scheme.h"
 #include "session.h"
 #include "task.h"
+#include "tx.h"
 
 #include "api/channel.h"
 #include "api/data.h"
 #include "api/log.h"
 #include "api/session.h"
 #include "api/task.h"
-
-static GStaticPrivate thread_application_context_key = G_STATIC_PRIVATE_INIT;
-static GStaticPrivate thread_identity_key = G_STATIC_PRIVATE_INIT;
+#include "api/util.h"
 
 static SCM scm_make_callback;
 static SCM scm_callback_module;
@@ -75,83 +76,6 @@ static SCM scm_make_object_removed_condition;
 static SCM scm_make_name_exists_condition;
 static SCM scm_make_name_not_bound_condition;
 
-static void *with_application_context 
-(gzochid_application_context *context, gzochid_auth_identity *identity,
- void *(*worker) (gpointer), gpointer data)
-{
-  gpointer ret = NULL;
-  gboolean private_needs_context = 
-    g_static_private_get (&thread_application_context_key) == NULL;
-
-  if (private_needs_context)
-    {
-      g_static_private_set (&thread_application_context_key, context, NULL);
-      g_static_private_set (&thread_identity_key, identity, NULL);
-    }
-  
-  ret = worker (data);
-
-  if (private_needs_context)
-    {
-      g_static_private_set (&thread_application_context_key, NULL, NULL);
-      g_static_private_set (&thread_identity_key, NULL, NULL);
-    }
-
-  return ret;
-}
-
-static SCM scheme_catch_body (void *data)
-{
-  void **ptr = (void **) data;
-
-  SCM procedure = (SCM) ptr[0];
-  SCM args = (SCM) ptr[1];
-  
-  return scm_apply_0 (procedure, args);
-}
-
-static SCM scheme_catch_handler (void *data, SCM tag, SCM throw_args)
-{
-  SCM exception_variable = (SCM) data;
-
-  if (scm_variable_p (exception_variable) == SCM_BOOL_T)
-    scm_variable_set_x (exception_variable, throw_args);
-
-  return SCM_BOOL_F;
-}
-
-static SCM scheme_pre_unwind_handler (void *data, SCM tag, SCM throw_args)
-{
-  scm_display_backtrace 
-    (scm_make_stack (SCM_BOOL_T, SCM_EOL),
-     scm_current_output_port (), 
-     SCM_BOOL_F, 
-     SCM_BOOL_F);
-
-  scm_print_exception (scm_current_output_port (), SCM_BOOL_F, tag, throw_args);
-
-  return SCM_BOOL_F;
-}
-
-static gpointer scheme_invoke_inner (gpointer data)
-{
-  void **ptr = (void **) data;
-  void *sub_data[2];
-
-  SCM procedure = (SCM) ptr[0];
-  SCM args = (SCM) ptr[1];
-  SCM exception = (SCM) ptr[2];
-
-  sub_data[0] = procedure;
-  sub_data[1] = args;
-
-  return scm_c_catch 
-    (SCM_BOOL_T,
-     scheme_catch_body, sub_data, 
-     scheme_catch_handler, exception, 
-     scheme_pre_unwind_handler, NULL);
-}
-
 static SCM resolve_procedure (char *procedure, GList *module)
 {
   SCM scm_procedure = scm_from_locale_symbol (procedure);
@@ -160,9 +84,20 @@ static SCM resolve_procedure (char *procedure, GList *module)
   return scm_public_ref (scm_module, scm_procedure);
 }
 
+static gpointer scheme_invoke_inner (gpointer data)
+{
+  void **ptr = (void **) data;
+
+  SCM procedure = (SCM) ptr[0];
+  SCM args = (SCM) ptr[1];
+  SCM exception_var = (SCM) ptr[2];
+
+  return gzochid_guile_invoke (procedure, args, exception_var);
+}
+
 SCM gzochid_scheme_invoke
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
- char *procedure, GList *module, SCM args, SCM exception)
+ char *procedure, GList *module, SCM args, SCM exception_var)
 {
   void *data[3];
   SCM ret = SCM_EOL;
@@ -185,19 +120,31 @@ SCM gzochid_scheme_invoke
 
   data[0] = resolve_procedure (procedure, module);
   data[1] = args;
-  data[2] = exception;
+  data[2] = exception_var;
 
-  ret = (SCM) with_application_context 
+  ret = (SCM) gzochid_with_application_context 
     (context, identity, scheme_invoke_inner, data);
   scm_variable_set_x (load_path, backup_load_path);
 
+  scm_remember_upto_here_1 (args);
+  scm_remember_upto_here_1 (exception_var);
+
   return ret;
 }
 
+static int scheme_prepare (gpointer data) { return FALSE; }
+static void scheme_commit (gpointer data) { assert (1 == 0); }
+static void scheme_rollback (gpointer data) { }
+
+static gzochid_transaction_participant scheme_participant = 
+  { "scheme", scheme_prepare, scheme_commit, scheme_rollback };
+
 void gzochid_scheme_application_worker 
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gpointer task)
 {
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+
   gzochid_scheme_invoke 
     (context, 
      identity,
@@ -206,7 +153,247 @@ void gzochid_scheme_application_worker
      (g_list_append 
       (g_list_append (NULL, "gzochi"), "private"), "task"), 
      scm_list_1 ((SCM) task), 
-     SCM_BOOL_F);
+     exception_var);
+
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    {
+      gzochid_transaction_join (&scheme_participant, NULL);
+      gzochid_transaction_mark_for_rollback (&scheme_participant);
+    }
+}
+
+static gzochid_application_callback *scm_to_callback 
+(gzochid_application_context *context, SCM scm_callback)
+{
+  GList *module = gzochid_scheme_callback_module (scm_callback);
+  char *procedure = gzochid_scheme_callback_procedure (scm_callback);
+  gzochid_data_managed_reference *reference = 
+    gzochid_data_create_reference 
+    (context, &gzochid_scheme_data_serialization, scm_callback);
+
+  mpz_t oid;
+  
+  mpz_init (oid);
+  mpz_set (oid, reference->oid);      
+    
+  return gzochid_application_callback_new (procedure, module, oid);
+}
+
+void gzochid_scheme_application_initialized_worker 
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ gpointer data)
+{
+  GHashTable *properties = (GHashTable *) data;
+  gzochid_data_managed_reference *callback_reference = NULL;
+  SCM callback = gzochid_scheme_create_callback 
+    (context->descriptor->initialized, NULL);
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+
+  callback_reference = gzochid_data_create_reference 
+    (context, &gzochid_scheme_data_serialization, callback);
+
+  gzochid_scheme_invoke 
+    (context,
+     identity,
+     "gzochi:execute-initialized",
+     g_list_append
+     (g_list_append
+      (g_list_append (NULL, "gzochi"), "private"), "app"),
+     scm_list_2 (callback,
+		 gzochid_scheme_ghashtable_to_hashtable 
+		 (properties, 
+		  gzochid_scheme_string_hash,
+		  gzochid_scheme_string_equiv,
+		  (SCM (*) (gpointer)) scm_from_locale_string,
+		  (SCM (*) (gpointer)) scm_from_locale_string)),
+     exception_var);
+
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    {
+      gzochid_transaction_join (&scheme_participant, NULL);
+      gzochid_transaction_mark_for_rollback (&scheme_participant);
+    }
+  else gzochid_data_set_binding_to_oid 
+	 (context, "s.initializer", callback_reference->oid);
+}
+
+static gpointer unpack_handler (gpointer data)
+{
+  SCM handler = (SCM) data;
+  gzochid_application_context *context = 
+    gzochid_get_current_application_context ();
+  gzochid_client_session_handler *session_handler = 
+    malloc (sizeof (gzochid_client_session_handler));
+
+  session_handler->received_message = scm_to_callback 
+    (context, gzochid_scheme_handler_received_message (handler));
+  session_handler->disconnected = scm_to_callback 
+    (context, gzochid_scheme_handler_disconnected (handler));
+
+  return session_handler;
+}
+
+void gzochid_scheme_application_logged_in_worker 
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ gpointer data)
+{
+  mpz_t session_oid;
+  char *session_oid_str = (char *) data;
+  gzochid_client_session *session = NULL;
+  gzochid_data_managed_reference *session_reference = NULL;
+
+  SCM scm_session = SCM_BOOL_F;
+  gzochid_data_managed_reference *session_scm_reference = NULL;
+
+  SCM cb = SCM_BOOL_F;
+  gzochid_data_managed_reference *callback_reference = NULL;
+
+  SCM handler = SCM_BOOL_F;
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+
+  mpz_init (session_oid);
+  mpz_set_str (session_oid, session_oid_str, 16);
+  session_reference = gzochid_data_create_reference_to_oid
+    (context, &gzochid_client_session_serialization, session_oid);
+  mpz_clear (session_oid);
+
+  gzochid_data_dereference (session_reference);
+  session = (gzochid_client_session *) session_reference->obj;
+
+  scm_session = gzochid_scheme_create_client_session 
+    (session, session_reference->oid);
+  session_scm_reference = 
+    gzochid_data_create_reference
+    (context, &gzochid_scheme_data_serialization, scm_session);
+
+  cb = gzochid_scheme_create_callback (context->descriptor->logged_in, NULL);
+  callback_reference = gzochid_data_create_reference 
+    (context, &gzochid_scheme_data_serialization, cb);
+
+  handler = gzochid_scheme_invoke 
+    (context,
+     identity,
+     "gzochi:execute-logged-in",
+     g_list_append
+     (g_list_append
+      (g_list_append (NULL, "gzochi"), "private"), "app"),
+     scm_list_2 ((SCM) callback_reference->obj, 
+		 (SCM) session_scm_reference->obj), 
+     exception_var);
+  
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    {
+      gzochid_transaction_join (&scheme_participant, NULL);
+      gzochid_transaction_mark_for_rollback (&scheme_participant);
+    }
+  else if (scm_is_false (handler))
+    gzochid_client_session_send_login_failure (context, session);
+  else
+    {
+      session->handler = (gzochid_client_session_handler *) 
+	gzochid_with_application_context 
+	(context, identity, unpack_handler, handler);
+
+      gzochid_data_mark 
+	(context, &gzochid_client_session_serialization, session);
+      gzochid_client_session_send_login_success (context, session);
+    }
+}
+
+void gzochid_scheme_application_received_message_worker 
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ gpointer ptr)
+{
+  gzochid_client_session *session = NULL;
+  gzochid_data_managed_reference *session_reference = NULL;
+  gzochid_data_managed_reference *callback_reference = NULL;
+
+  SCM bv = SCM_BOOL_F;
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+  GList *gpa = g_list_append
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
+
+  unsigned char *message = NULL;
+  short *message_len_ptr = NULL;
+  void **data = (void **) ptr;
+
+  mpz_t session_oid;
+  mpz_init (session_oid);
+  mpz_set_str (session_oid, (char *) data[0], 16);
+
+  message = (unsigned char *) data[1];
+  message_len_ptr = (short *) data[2];
+
+  session_reference = gzochid_data_create_reference_to_oid 
+    (context, &gzochid_client_session_serialization, session_oid);
+  gzochid_data_dereference (session_reference);
+  session = (gzochid_client_session *) session_reference->obj;
+
+  bv = gzochid_scheme_create_bytevector (message, *message_len_ptr);
+  callback_reference =
+    gzochid_data_create_reference_to_oid
+    (context, &gzochid_scheme_data_serialization, 
+     session->handler->received_message->scm_oid);
+  gzochid_data_dereference (callback_reference);
+      
+  gzochid_scheme_invoke
+    (context, 
+     identity,
+     "gzochi:execute-received-message", gpa,
+     scm_list_2 ((SCM) callback_reference->obj, bv),
+     exception_var);
+
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    {
+      gzochid_transaction_join (&scheme_participant, NULL);
+      gzochid_transaction_mark_for_rollback (&scheme_participant);
+    }
+
+  g_list_free (gpa);
+}
+
+void gzochid_scheme_application_disconnected_worker 
+(gzochid_application_context *context, gzochid_auth_identity *identity, 
+ gpointer ptr)
+{
+  gzochid_client_session *session = NULL;
+  gzochid_data_managed_reference *session_reference = NULL;
+  gzochid_data_managed_reference *callback_reference = NULL;
+
+  SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+  GList *gpa = g_list_append
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
+
+  mpz_t session_oid;
+  mpz_init (session_oid);
+  mpz_set_str (session_oid, (char *) ptr, 16);
+
+  session_reference = gzochid_data_create_reference_to_oid 
+    (context, &gzochid_client_session_serialization, session_oid);
+  gzochid_data_dereference (session_reference);
+  session = (gzochid_client_session *) session_reference->obj;
+
+  callback_reference =
+    gzochid_data_create_reference_to_oid
+    (context, &gzochid_scheme_data_serialization, 
+     session->handler->disconnected->scm_oid);
+  gzochid_data_dereference (callback_reference);
+      
+  gzochid_scheme_invoke
+    (context, 
+     identity,
+     "gzochi:execute-disconnected", gpa,
+     scm_list_1 ((SCM) callback_reference->obj),
+     exception_var);
+
+  if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
+    {
+      gzochid_transaction_join (&scheme_participant, NULL);
+      gzochid_transaction_mark_for_rollback (&scheme_participant);
+    }
+  else gzochid_data_remove_object (session_reference);
+
+  g_list_free (gpa);
 }
 
 static void scheme_worker_serializer 
@@ -584,8 +771,14 @@ SCM gzochid_scheme_create_managed_reference
 
   SCM scm_oid = scm_string_to_number 
     (scm_from_locale_string (oid_str), scm_from_short (16));
-  SCM data = reference->obj == NULL ? SCM_BOOL_F : (SCM) reference->obj;
-  SCM ret = scm_call_2 (scm_make_managed_reference, scm_oid, data);
+  SCM data = SCM_BOOL_F;
+  SCM ret = SCM_BOOL_F;
+
+  if (reference->obj != NULL)
+    data = gzochid_scm_location_resolve 
+      (reference->context, (gzochid_scm_location_info *) reference->obj);
+  
+  ret = scm_call_2 (scm_make_managed_reference, scm_oid, data);
 
   free (oid_str);
   scm_gc_protect_object (ret);
@@ -705,6 +898,7 @@ static void *initialize_bindings (void *ptr)
   gzochid_api_log_init ();
   gzochid_api_session_init ();
   gzochid_api_task_init ();
+  gzochid_api_util_init ();
   
   return NULL;
 }
@@ -735,19 +929,3 @@ void gzochid_scheme_initialize_bindings (void)
 {
   scm_with_guile (initialize_bindings, NULL);
 }
-
-gzochid_application_context *gzochid_scheme_current_application_context (void)
-{
-  gzochid_application_context *context = (gzochid_application_context *)
-    g_static_private_get (&thread_application_context_key);
-  assert (context != NULL);
-  return context;
-}
-
-gzochid_auth_identity *gzochid_scheme_current_identity (void)
-{
-  gzochid_auth_identity *identity = (gzochid_auth_identity *)
-    g_static_private_get (&thread_identity_key);
-  assert (identity != NULL);
-  return identity;
-}
