@@ -1,5 +1,5 @@
 ;; gzochi.scm: The gzochi composite library
-;; Copyright (C) 2011 Julian Graham
+;; Copyright (C) 2012 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
   (export ;; (gzochi app)
 
           gzochi:make-callback gzochi:callback? gzochi:callback-module
-	  gzochi:callback-procedure gzochi:callback-data
+	  gzochi:callback-procedure gzochi:callback-data 
+	  %gzochi:application-root
    
           ;; (gzochi channel)
 
@@ -34,7 +35,8 @@
 	  gzochi:client-session? gzochi:client-session-name 
 	  gzochi:client-session-listener gzochi:make-client-session-listener
 	  gzochi:client-session-listener-received-message
-	  gzochi:client-session-listener-disconnected
+	  gzochi:client-session-listener-disconnected gzochi:send-message
+	  gzochi:disconnect
 
           ;; (gzochi conditions)
 
@@ -47,10 +49,29 @@
 	  gzochi:make-managed-record-type-descriptor
 	  gzochi:managed-record-accessor gzochi:managed-record-mutator
 	  gzochi:managed-record-constructor gzochi:managed-record-predicate
-	  gzochi:managed-record? gzochi:define-managed-record-type
+	  gzochi:managed-record? gzochi:managed-record-rtd 
+	  gzochi:managed-record-type-name gzochi:managed-record-type-parent 
+	  gzochi:managed-record-type-uid gzochi:define-managed-record-type 
 	  gzochi:managed-record-type-descriptor 
 	  gzochi:managed-record-constructor-descriptor gzochi:get-binding 
-	  gzochi:set-binding! gzochi:remove-binding!
+	  gzochi:set-binding! gzochi:remove-binding! gzochi:remove-object!
+
+	  gzochi:make-managed-serializable gzochi:managed-serializable?
+	  gzochi:managed-serializable-value
+	  gzochi:managed-serializable-value-set!
+
+	  gzochi:make-managed-vector gzochi:managed-vector?
+	  gzochi:managed-vector-ref gzochi:managed-vector-set!
+          gzochi:managed-vector-length gzochi:managed-vector->list
+
+          gzochi:make-managed-hashtable gzochi:managed-hashtable?
+          gzochi:managed-hashtable-size gzochi:managed-hashtable-ref
+          gzochi:managed-hashtable-set! gzochi:managed-hashtable-delete!
+          gzochi:managed-hashtable-contains? gzochi:managed-hashtable-update!
+          gzochi:managed-hashtable-clear! gzochi:managed-hashtable-keys
+          gzochi:managed-hashtable-entries
+	  gzochi:managed-hashtable-hash-function
+          gzochi:managed-hashtable-equivalence-function
 
           ;; (gzochi io)
 
