@@ -18,20 +18,23 @@
 
 (library (gzochi private data)
   (export gzochi:make-managed-record-type-descriptor
+	  gzochi:make-managed-record-constructor-descriptor
 	  gzochi:managed-record-accessor
 	  gzochi:managed-record-mutator
 	  gzochi:managed-record-constructor
 	  gzochi:managed-record-predicate
 
-	  gzochi:managed-record?
-
           gzochi:define-managed-record-type
 	  gzochi:managed-record-type-descriptor
 	  gzochi:managed-record-constructor-descriptor
 
-	  gzochi:managed-record
 	  gzochi:make-managed-record
+
 	  gzochi:managed-record?
+	  gzochi:managed-record-rtd 
+	  gzochi:managed-record-type-name
+	  gzochi:managed-record-type-parent 
+	  gzochi:managed-record-type-uid
 
 	  gzochi:create-reference
 	  gzochi:dereference
@@ -101,6 +104,11 @@
   (define gzochi:managed-record-constructor record-constructor)
   (define gzochi:managed-record-predicate record-predicate)
 
+  (define gzochi:managed-record-rtd record-rtd)
+  (define gzochi:managed-record-type-name record-type-name)
+  (define gzochi:managed-record-type-parent record-type-parent)
+  (define gzochi:managed-record-type-uid record-type-uid)
+
   (define (managed-record-subtype? rtd)
     (or (eq? rtd gzochi:managed-record)
 	(and=> (record-type-parent rtd) managed-record-subtype?)))
@@ -528,10 +536,7 @@
   (define-record-type (gzochi:managed-reference 
 		       gzochi:make-managed-reference 
 		       gzochi:managed-reference?)
-    (fields oid 
-	    (mutable record 
-		     gzochi:managed-reference-record 
-		     gzochi:managed-reference-set-record!)))
+    (fields oid))
 
   (define (gzochi:create-reference obj)
     (or (gzochi:managed-record? obj)
@@ -547,11 +552,7 @@
 	(raise (condition (make-assertion-violation)
 			  (make-message-condition
 			   "Only managed references can be dereferenced."))))
-    (let ((record (gzochi:managed-reference-record reference)))
-      (or record
-	  (let ((data (primitive-dereference reference)))
-	    (gzochi:managed-reference-set-record! reference data)
-	    data))))
+    (primitive-dereference reference))
 
   (define (gzochi:get-binding name)
     (or (string? name) (raise (make-assertion-violation)))
